@@ -1,10 +1,13 @@
 from threading import Semaphore
-from typing import Dict, List, TypeVar, Type, Optional
+from typing import Dict, TypeVar, Type, Optional
 
 import numpy as np
 
 from dt_duckiematrix_protocols.commons.LayerProtocol import LayerProtocol
-from dt_duckiematrix_protocols.utils.Pose3D import Pose3D
+from dt_duckiematrix_protocols.commons.ProtocolAbs import ProtocolAbs
+from dt_duckiematrix_protocols.types.colors import IColor, LocalColor
+from dt_duckiematrix_protocols.types.geometry import LocalVector3, IVector3, LocalPose3D, IPose3D
+from dt_duckiematrix_protocols.types.markers import IMarker
 from dt_duckiematrix_protocols.viewer.markers import \
     MarkerAbs, \
     MarkerAction, \
@@ -14,7 +17,10 @@ from dt_duckiematrix_protocols.viewer.markers import \
     MarkerQuad, \
     MarkerArrow, \
     MarkerText, \
-    MarkerTrajectory
+    MarkerTrajectory, \
+    MarkerSimple, \
+    MarkerMultipart, \
+    MarkerCone
 
 T = TypeVar('T')
 
@@ -23,7 +29,7 @@ class MarkersManager:
 
     def __init__(self, engine_hostname: str, auto_commit: bool = False,
                  layer_protocol: Optional[LayerProtocol] = None):
-        self._markers: Dict[str, MarkerAbs] = {}
+        self._markers: Dict[str, IMarker] = {}
         self._markers_actions: Dict[str, MarkerAction] = {}
 
         # protocols
@@ -36,16 +42,30 @@ class MarkersManager:
         self._auto_commit = auto_commit
         self._lock = Semaphore(1)
 
-    def _add(self, key: str, marker: MarkerAbs):
+    def session(self) -> ProtocolAbs.SessionProtocolContext:
+        return self._layer_protocol.session()
+
+    def atomic(self) -> ProtocolAbs.AtomicProtocolContext:
+        return self._layer_protocol.atomic()
+
+    def _add(self, key: str, marker: IMarker):
         # TODO: not sure what happens when markers are replaced with different types
         # issue a REMOVE action when another marker with the same key already exists
         if key in self._markers:
             self._remove(key)
         # add new marker
         with self._lock:
-            self._markers[key] = marker
-            self._markers_actions[key] = MarkerAction.ADD_OR_UPDATE
-            self._layer_protocol.update("markers", key, marker.as_dict())
+            if isinstance(marker, MarkerSimple):
+                self._markers[key] = marker
+                self._markers_actions[key] = MarkerAction.ADD_OR_UPDATE
+                self._layer_protocol.update("markers", key, marker.as_dict())
+            elif isinstance(marker, MarkerMultipart):
+                for k, m in marker.as_dict().items():
+                    self._markers[k] = m
+                    self._markers_actions[k] = MarkerAction.ADD_OR_UPDATE
+                    self._layer_protocol.update("markers", k, m)
+            else:
+                raise ValueError(f"Marker of type {type(marker).__name__} not recognized.")
 
     def _remove(self, key: str):
         with self._lock:
@@ -56,12 +76,12 @@ class MarkersManager:
             self._markers_actions[key] = MarkerAction.REMOVE
             self._layer_protocol.update("markers", key, marker.as_dict())
 
-    def _make_pose(self, key: str, auto_commit: bool = False, **kwargs) -> Pose3D:
-        return Pose3D(self._layer_protocol, key, auto_commit, **kwargs)
+    @staticmethod
+    def _make_pose(**kwargs) -> IPose3D:
+        return LocalPose3D(**kwargs)
 
     @staticmethod
-    def _make_scale(auto_commit: bool = False, **kwargs) -> List[float]:
-        # TODO: this should return a monitored object
+    def _make_scale(**kwargs) -> Optional[IVector3]:
         scale_vector = None
         if "scale" in kwargs:
             scale = kwargs["scale"]
@@ -77,11 +97,12 @@ class MarkersManager:
                 scale_vector = list(map(float, scale))
             else:
                 raise ValueError(f"Object '{scale}' is not a valid scale.")
-        return scale_vector
+        if scale_vector is not None:
+            data = dict(zip(["x", "y", "z"], scale_vector))
+            return LocalVector3(**data)
 
     @staticmethod
-    def _make_color(auto_commit: bool = False, **kwargs) -> List[float]:
-        # TODO: this should return a monitored object
+    def _make_color(**kwargs) -> Optional[IColor]:
         color_vector = None
         if "color" in kwargs:
             color = kwargs["color"]
@@ -99,27 +120,29 @@ class MarkersManager:
                 raise ValueError(f"Object '{color}' is not a valid color.")
         if color_vector is not None and len(color_vector) == 3:
             color_vector = color_vector + [1.0]
-        return color_vector
+        if color_vector is not None:
+            data = dict(zip(["r", "g", "b", "a"], color_vector))
+            return LocalColor(**data)
 
     def _make_marker(self, key: str, factory: Type[T], **kwargs) -> T:
         auto_commit: bool = self._auto_commit
-        marker = factory(self._layer_protocol, key, auto_commit)
-        # pose
-        pose = self._make_pose(key, auto_commit, **kwargs)
-        if pose is not None:
-            marker.pose = pose
-        # scale
-        scale = self._make_scale(auto_commit, **kwargs)
-        if scale is not None:
-            marker.scale = scale
-        # color
-        color = self._make_color(auto_commit, **kwargs)
-        if color is not None:
-            marker.color = color
-        # action
-        marker.action = MarkerAction.ADD_OR_UPDATE
-        # add marker
-        self._add(key, marker)
+        with self._layer_protocol.atomic():
+            marker: MarkerAbs = factory(self._layer_protocol, key, auto_commit, **kwargs)
+            # pose
+            pose = self._make_pose(**kwargs)
+            marker.pose.update(pose)
+            # scale
+            scale = self._make_scale(**kwargs)
+            if scale is not None:
+                marker.scale.update(scale)
+            # color
+            color = self._make_color(**kwargs)
+            if color is not None:
+                marker.color.update(color)
+            # action
+            marker.action = MarkerAction.ADD_OR_UPDATE
+            # add marker
+            self._add(key, marker)
         # ---
         return marker
 
@@ -135,9 +158,11 @@ class MarkersManager:
     def Quad(self, key: str, **kwargs) -> MarkerQuad:
         return self._make_marker(key, MarkerQuad, **kwargs)
 
+    def Cone(self, key: str, **kwargs) -> MarkerCone:
+        return self._make_marker(key, MarkerCone, **kwargs)
+
     def Arrow(self, key: str, **kwargs) -> MarkerArrow:
         return self._make_marker(key, MarkerArrow, **kwargs)
-        # TODO: add custom fields setup
 
     def Text(self, key: str, **kwargs) -> MarkerText:
         return self._make_marker(key, MarkerText, **kwargs)
